﻿local playerEnteredBG = false;
rafMessageDisplayed = false; -- Temporary. Used for the RAF beta message.

----------------------
-- Event callbacks
----------------------
function MainOnload()
    -- Register Events
    this:RegisterEvent("CHAT_MSG_COMBAT_XP_GAIN");
    this:RegisterEvent("PLAYER_LOGIN");
    this:RegisterEvent("PLAYER_LEVEL_UP");
    this:RegisterEvent("PLAYER_XP_UPDATE");
	this:RegisterEvent("UNIT_PET");
	this:RegisterEvent("UNIT_PET_EXPERIENCE");
	this:RegisterEvent("PET_UI_UPDATE");
	this:RegisterEvent("PLAYER_ENTERING_BATTLEGROUND");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("CHAT_MSG_BG_SYSTEM_NEUTRAL");
	
	this:RegisterEvent("ZONE_CHANGED_INDOORS");
	this:RegisterEvent("ZONE_CHANGED_NEW_AREA");
	this:RegisterEvent("ZONE_CHANGED");
    
    -- Register slash commands
    SLASH_XTOLEVEL1 = "/xtolevel";
    SLASH_XTOLEVEL2 = "/xtl";
    SlashCmdList["XTOLEVEL"] = OnSlashCommand;
end

function MainOnPlayerLogin()
	VerifyConfig()

    local cDisplayLocale = sConfig.general.displayLocale or DISPLAY_LOCALE;
	L = LOCALE[cDisplayLocale];
    if L == nil then
        cDisplay:Debug("Attempted to load unknow locale '" .. tostring(cDisplayLocale) .."'. Falling back on 'enUS'.");
        L = LOCALE["enUS"];
        sConfig.general.displayLocale = "enUS";
    end

    cPlayer:Initialize(sData.player.killAverage, sData.player.questAverage)
	cPet:Initialize(sData.pet.killAverage)
    cDisplay.Average:Initialize()
	cDisplay.LDB:Initialize()
    --cDisplay.Config:Initialize()
	
	--XToLevelConfig:Hide();
end

function MainOnEnter()
	if sConfig.averageDisplay.tooltip then
		local footer = (sConfig.general.allowSettingsClick and L['Right Click To Configure']) or nil
		cDisplay.Tooltip:Show(nil, nil, nil, nill, footer)
	end
end
function MainOnLeave()
	cDisplay.Tooltip:Hide();
end

function OnPlayerLevelUp(newLevel)
    cDisplay.Floating:PrintLevel(newLevel)
    cDisplay.Chat:PrintLevel(newLevel)
	
	cPlayer.level = newLevel
	cPet:Update()
	
	cDisplay.Average:Update()
    
    cDisplay.LDB:BuildPattern();
	cDisplay.LDB:Update();
end

function OnChatXPGain(message)
    -- Note that this event is fired by kills, quests and BG objectives.
    local xp, mobName = ParseChatXPMessage(message, LOCALE_CLIENT)
    xp = tonumber(xp)
    
    -- See if it is a kill or a quest
    if mobName ~= nil then
		if playerEnteredBG then
			cDisplay:Debug("Battleground Kill detected: " .. tostring(xp) .. "(" .. mobName ..")")
			cPlayer:AddBattlegroundKill(xp, mobName)
		else
			local unrestedXP = cPlayer:AddKill(xp, mobName)
			
			if sConfig.messages.bgObjectives and (sConfig.messages.playerFloating or sConfig.messages.playerChat) then
				local killsRequired = cPlayer:GetKillsRequired(unrestedXP)
				if killsRequired > 0 then
					cDisplay.Floating:PrintKill(mobName, ceil(killsRequired / ( (IsRafApplied() and 3) or 1 )))
					cDisplay.Chat:PrintKill(mobName, killsRequired)
					cPet.nextMobName = mobName -- Sets the next mob to be displayed for the pet message.
				end
			end
		end
    else
		if playerEnteredBG then
			cDisplay:Debug("Objective XP gained! : " .. tostring(xp))
			local isObj = cPlayer:AddBattlegroundObjective(xp)
			if isObj and cPlayer.isActive then
				local objectivesRequired = cPlayer:GetQuestsRequired(xp)
				if objectivesRequired > 0 then
					cDisplay.Floating:PrintBGObjective(objectivesRequired)
					cDisplay.Chat:PrintBGObjective(objectivesRequired)
				end
			end
		else
			cPlayer:AddQuest(xp)
			if sConfig.messages.playerFloating or sConfig.messages.playerChat then
				local questsRequired = cPlayer:GetQuestsRequired(xp)
				if questsRequired > 0 then
					cDisplay.Floating:PrintQuest( ceil(questsRequired / ( (IsRafApplied() and 3) or 1 )) )
					cDisplay.Chat:PrintQuest(questsRequired)
				end
			end
		end
    end
end

function OnPlayerXPUpdate()
    cPlayer:SyncData()
    cDisplay.Average:Update()
    cDisplay.LDB:BuildPattern();
	cDisplay.LDB:Update()
    
    sData.player.killAverage = cPlayer:GetAverageKillXP()
    sData.player.questAverage = cPlayer:GetAverageQuestXP()
end

function OnUnitPet(arg1)
    -- Note, it appears this event is now fired before the PLAYER_LOGIN event
    -- so the player won't be initialized the first time it is fired. (regression bugs wtf!)
	if arg1 == "player" and cPlayer.level ~= nil then
		cPet:Initialize()
		cDisplay.Average:Update()
        cDisplay.LDB:BuildPattern();
		cDisplay.LDB:Update()
	end
end

function OnUnitPetExperience()
	if not playerEnteredBG and cPet.isActive then
		local update, killsRemaining, mobName;
		update = cPet:Update()
        cDisplay:Debug('update.xp: ' .. tostring(update.xp))
        cDisplay:Debug('update.gainedLevel: ' .. tostring(update.gainedLevel))
		if update.xp > 0 then
			killsRemaining = ceil((cPet.maxXP - cPet.xp) / update.xp)
			if killsRemaining > 0 then
				mobName = cPet:GetMobName()
				cDisplay.Floating:PrintPetKill(cPet:GetName(), mobName, killsRemaining)
				cDisplay.Chat:PrintPetKill(cPet:GetName(), mobName, killsRemaining)
			end
		end
        if update.gainedLevel then
            cDisplay.Floating:PrintPetLevel(cPet:GetName())
            cDisplay.Chat:PrintPetLevel(cPet:GetName())
        end
		cDisplay.Average:Update()
        cDisplay.LDB:BuildPattern()
		cDisplay.LDB:Update()
	end
end

function OnPetUiUpdate()
	-- This event only fires after a new pet has been trained (as far as I know
	-- Included here to circumvent a problem with the UNIT_PET event firing to early when a new pet is trained.
	cPet:Initialize()
	cDisplay.Average:Update()
    cDisplay.LDB:BuildPattern();
	cDisplay.LDB:Update()
end

function PlayerEnteringBG()
	playerEnteredBG = true
	if cPlayer.isActive then
		cPlayer:BattlegroundStart(false)
	else
		cDisplay:Debug("Entered BG. Player counter inactive. Count cancelled.")
	end
end
function PlayerEnteringWorld()
	if playerEnteredBG then
		playerEnteredBG = false
		
		if cPlayer.isActive then
			local bgsRequired = cPlayer:GetQuestsRequired(sData.player.bgList[1].totalXP)
			cPlayer:BattlegroundEnd()
			
			cDisplay.Floating:PrintBattleground(bgsRequired)
			cDisplay.Chat:PrintBattleground(bgsRequired)
			cDisplay.Average:Update()
            cDisplay.LDB:BuildPattern();
			cDisplay.LDB:Update()
		else
			cDisplay:Debug("Left BG. Player inactive. Nothing to do.")
		end
	end
end

function AreaChanged()
	if playerEnteredBG and cPlayer.isActive then
		local oldZone = sData.player.bgList[1].name
		local newZone = GetRealZoneText()
		if oldZone == false then
			sData.player.bgList[1].name = newZone
			cDisplay:Debug(" - BG name set. ")
		else
			if oldZone ~= newZone then
				cDisplay:Debug(" - BG names don't match (" .. oldZone .." vs " .. newZone .."). Restarting.")
				cPlayer:BattlegroundEnd()
				cPlayer:BattlegroundStart(newZone)
			end
		end
	end
end

function OnSlashCommand(arg1)

    

	if arg1 == "clear kills" then
		cPlayer:ClearKillList()
		cPlayer.killAverage = nil
		cDisplay:Print("Player kill records cleared.")
		cDisplay.Average:Update()
        cDisplay.LDB:BuildPattern();
		cDisplay.LDB:Update()
	elseif arg1 == "clear quests" then
		cPlayer:ClearQuestList()
		cPlayer.questAverage = nil
		cDisplay:Print("Player quests records cleared.")
		cDisplay.Average:Update()
        cDisplay.LDB:BuildPattern();
		cDisplay.LDB:Update()
	elseif arg1 == "clear bg" then
		cPlayer:ClearBattlegroundList()
		cPlayer.bgAverage = nil
		cPlayer.bgObjAverage = nil
		cDisplay:Print("Player battleground records cleared.")
		cDisplay.Average:Update()
        cDisplay.LDB:BuildPattern();
		cDisplay.LDB:Update()
	elseif arg1 == "d" then
		for petName, petList in pairs(sData.pet.killList) do
			cDisplay:Debug("List for: " .. petName)
			for killIndex, killXP in ipairs(petList) do
				cDisplay:Debug(" #"..killIndex..": "..killXP)
			end
		end
	else
        XToLevelConfig:Show();
        XToLevelConfig_SwapPanels('Messages');
	end
end

----------------------
-- Main event handler
----------------------
function MainOnEvent()
	if event == "PLAYER_LOGIN" then
		MainOnPlayerLogin()
	elseif event == "PLAYER_LEVEL_UP" then
		OnPlayerLevelUp(arg1)
	elseif event == "CHAT_MSG_COMBAT_XP_GAIN" then
		OnChatXPGain(arg1)
	elseif event == "PLAYER_XP_UPDATE" then
		OnPlayerXPUpdate()
	elseif (event == "UNIT_PET") then
		OnUnitPet(arg1)
	elseif event == "UNIT_PET_EXPERIENCE" then
		OnUnitPetExperience();
	elseif event == "PET_UI_UPDATE" then
		OnPetUiUpdate()
	elseif event == "PLAYER_ENTERING_BATTLEGROUND" then
		PlayerEnteringBG()
	elseif event == "PLAYER_ENTERING_WORLD" then
		PlayerEnteringWorld()
	elseif event == "ZONE_CHANGED_NEW_AREA" or event == "ZONE_CHANGED_INDOORS" or event == "ZONE_CHANGED" then
		AreaChanged()
	end
end
